import requests
from requests_oauthlib import OAuth2Session
from flask import redirect, request
from .configs import *

class Oauth:

    def __init__(self, flask_discord):
        self.client_id = flask_discord.app.config["DISCORD_CLIENT_ID"]
        self.client_secret = flask_discord.app.config["DISCORD_CLIENT_SECRET"]
        self.redirect_uri = flask_discord.app.config["DISCORD_OAUTH_REDIRECT_URI"]
        self.scope = flask_discord.app.config["DISCORD_OAUTH_SCOPE"]
        self.discord_login_url = self._make_login_url()
        self.discord_token_url = DISCORD_TOKEN_URL
        self.discord_api_url = DISCORD_API_BASE_URL


    def _make_login_session(self):
        return OAuth2Session( 
        client_id = self.client_id,
        scope = self.scope,
        redirect_uri = self.redirect_uri
        )


    def _make_login_url(self):
        return self._make_login_session().authorization_url(DISCORD_AUTHORIZATION_BASE_URL)[0]


    def get_login_url(self):
        return self.discord_login_url


    def redirect_login(self):
        return redirect(self.discord_login_url)


    def _get_access_token(self):
        auth = self._make_login_session().fetch_token(
                self.discord_token_url,
                client_secret = self.client_secret,
                authorization_response = request.url
            )
        return auth["access_token"]


    def _get_user_json(self, access_token):
        url = f"{self.discord_api_url}/users/@me"
        headers = {"Authorization": f"Bearer {access_token}"}
 
        user_object = requests.get(url = url, headers = headers).json()
        return user_object


    def callback(self):
        access = self._get_access_token()

        return self._get_user_json(access)


from setuptools import setup, find_packages


classifiers=[
    'Framework :: Flask',
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python'
]


requirements = [
    "Flask",
    "oauthlib",
    "requests_oauthlib",
]


setup(
    name='Flask-Discord-Extended',
    version='0.0.1',
    url='https://github.com/davidg0022/Flask-Discord-Extended',
    license='MIT',
    author='Gatea David',
    author_email='davidgatea21@gmail.com',
    description='The Python OAuth2 and Discord Bot Commands for Flask applications.',
    long_description_content_type="text/markdown",
    long_description=open('README.md').read() + '\n\n' + open('CHANGELOG.txt').read(),
    packages=find_packages(),
    platforms='any',
    zip_safe=False,
    include_package_data=True,
    install_requires=requirements,
    classifiers=classifiers
)


import requests
import json
from .configs import DISCORD_API_BASE_URL

from . import exceptions

class Utilities():

    def __init__(self, flask_app):
        self.authorization = flask_app.app.config["DISCORD_AUTHORIZATION"]
        self.headers = self._make_headers(self.authorization)

    def _make_headers(self, authorization = ""):
        headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            "authorization" : f"Bot {authorization}"
        }
        return headers
    
    def _request(self, route, method, payload = None):
        route = DISCORD_API_BASE_URL + route
        if payload == None:
            response = requests.request(method, route, headers = self.headers)
        else:
            response = requests.request(method, route, data = json.dumps(payload), headers = self.headers)

        if response.status_code == 401:
            raise exceptions.Unauthorized()
        
        if response.status_code == 429:
            raise exceptions.RateLimited(response.json(), response.headers)

        return response.json()

    def send_chanel_message(self, content, chanel):
        payload = {
            "content": content,
            # "embeds":[{
            #     "title":"Test",
            #      'color':0x0099ff,
            #      'fields':[
            #         {"name":"Regular field title", "value":"test","inline":True},
            #         {"name":"Regular field title", "value":"test","inline":True},
            #         {"name":"Regular field title", "value":"test","inline":True},
            #         {"name":"Regular field title", "value":"test"},
            #         {"name":"Regular field title", "value":"test","inline":True},
            #         {"name":"Regular field title", "value":"test","inline":True},
            #         ],
            #         "image":{
            #             "url":"https://i.imgur.com/AfFp7pu.png"
            #         }
            #         "footer":{
                        
            #         }
            #      }]
        }
        return self._request(route = f"/channels/{chanel}/messages", method = "POST", payload = payload)


    def get_guild_member_data(self, guild, user_id):
        return self._request(route = f"/guilds/{guild}/members/{user_id}", method = "GET")
    

    def add_role(self, guild, user_id, role):
        try:
            roles = self.get_guild_member_data(guild, user_id)["roles"]
        except KeyError:
            return {}
        if role not in roles:
            roles.append(role)
        payload = {
            "roles": roles
        }
        return self._request(route = f"/guilds/{guild}/members/{user_id}", method = "PATCH", payload = payload)


    def remove_role(self, guild, user_id, role):
        try:
            roles = self.get_guild_member_data(guild, user_id)["roles"]
        except KeyError:
            return {}
        if role in roles:
            roles.remove(role)
        payload = {
            "roles": roles
        }
        return self._request(route = f"/guilds/{guild}/members/{user_id}", method = "PATCH", payload = payload)

    
    def set_nickname(self, guild, user_id, nickname):
        payload = {
            "nick":nickname
        }
        return self._request(route = f"/guilds/{guild}/members/{user_id}", method = "PATCH", payload = payload)
    
    def set_chanel_name(self, chanel, name):
        payload = {
            "name": name
        }
        return self._request(route = f"/channels/{chanel}", method = "PATCH", payload = payload)

    
    def send_dm(self, user_id, content):
        payload = {
            "recipients" : [user_id]
        }
        try:
            chanel = self._request(route = "/users/@me/channels", method = "POST", payload = payload)["id"]
        except KeyError:
            return {}
        return self.send_chanel_message(chanel = chanel, content = content )
        


import warnings
from .oauth import Oauth
from .utilities import Utilities


class FlaskDiscord(object):

    def __init__(self, app = None):
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        self.app = app

        required_keys = ['DISCORD_CLIENT_ID', 'DISCORD_CLIENT_SECRET']

        oauth_keys = ['DISCORD_OAUTH_REDIRECT_URI', 'DISCORD_OAUTH_SCOPE']

        for key in required_keys:
            if key not in app.config:
                warnings.warn(f'{key} is not set.')

        if 'DISCORD_OAUTH' in app.config and app.config['DISCORD_OAUTH'] == True:
            for key in oauth_keys:
                if key not in app.config:
                    warnings.warn(f'{key} is not set.')
            self.Oauth = Oauth(self)

        self.Utilities = Utilities(self)

